#!perl

package This;
use warnings;
use strict;
use Test::More;

use XML::LibXML;

use DateTime;

my $parser = XML::LibXML->new;
my $rngschema = XML::LibXML::RelaxNG->new( location => "/usr/local/nagios/share/xml/opsview.rng" );

use LWP::UserAgent;
my $ua = LWP::UserAgent->new;

sub test {
    my ( $class, $req ) = @_;
    ok( my $res = $ua->request($req), "Request" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $res->content_type, "text/xml", "XML returned" );
    return $res;
}

package main;
use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";

use Opsview::Web::Test;

use Test::More qw(no_plan);

use HTTP::Request::Common;
use HTTP::Request;
use POSIX qw( strftime );

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";
use Opsview;
use Opsview::Config;

use Sys::Hostname;
my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# The test db has a slave monitoringcluster - we delete it here
{

    # Need to delete the foreign key first, based on the role
    Opsview->db_Main->do("DELETE FROM roles_monitoringservers");

    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<monitoringcluster action="delete" by_id="2" />
			</opsview>},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Returned OK status" );
}

# The test db also has a slave cluster - discard it because the reload fails
{

    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<monitoringcluster action="delete" by_id="3" />
			</opsview>},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Returned OK status" );
}

my $starttime = time;

# Reload
{
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<system action="reload"/>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/.*<detail>Master Monitoring Server:warning:Unable to lookup IP address for .*<status>WARNING</status>/s", "XML right" );
}

sleep 5;    # Allow time for backup to be created

# Get the backup id
my ( $auditlogid, $reloadid ) = Opsview->db_Main->selectrow_array("SELECT id, reloadid FROM auditlogs WHERE reloadid IS NOT NULL ORDER BY id DESC LIMIT 1");
die "Backup not found" unless $reloadid;

note("For backup, auditlogid=$auditlogid reloadid=$reloadid") if ( $ENV{TEST_VERBOSE} );

my $backupfile = Opsview::Config->backupfile($reloadid);
my $modtime    = ( stat $backupfile )[9];
cmp_ok( $starttime, "<", $modtime, "Backup file created successfully" );

# Note: Check for backup restore is done in 605-backup.t

# Check requires XML
{
    my $req = HTTP::Request->new(
        "PUT" => "http://$hostname/api",
        [qw(Content-Type text/xml)],
    );
    ok( my $res = $ua->request($req), "Request" );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/XML data required.*OPTIONS-ERROR/s", "text error message" );
}

# Invalid XML test
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<host clone="54"><name>bob</name></host><invalid>},
        )
    );
    ok( $res->is_error, "Errored 400" );
    is( $res->code, 400, "status 400" );
    like( $res->content, '/<detail>.*Extra content at the end of the document.*<\/detail>.*<status>XML-ERROR<\/status>.*/s', "Returned expected XML" );
}

# XML validation error
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<host clone="54"><name2>bob</name2></host>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Expecting element opsview, got host\n</detail>.*<status>XML-INVALID</status>/s", "Returned expected XML" );
}

# Subtle xml validation error
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview><host action="create"><name2>bob</name2></host></opsview>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Element host has extra content: name2.*</detail>.*<status>XML-INVALID</status>/s", "Returned OK status" );
}

# Check action attribute is given
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host><name>bob</name></host></opsview>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>No action specified</detail>.*<status>OPTIONS-ERROR</status>/s", "Returned OK status" );
}

# Check action attribute is not rubbish
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview><host action="rubbish"><name>bob</name></host></opsview>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Invalid attribute action for element host.*</detail>.*<status>XML-INVALID</status>/s", "Returned OK status" );
}

# ...and is not an inappropriate one
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview><host action="reload"><name>bob</name></host></opsview>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Invalid attribute action for element host.*</detail>.*<status>XML-INVALID</status>/s", "Returned OK status" );
}

# Check that action="delete" must give a by_id or by_name
{
    my $res = This->test(
        HTTP::Request->new(
            PUT => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview><host action="delete"><name2>bob</name2></host></opsview>},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Element host has extra content: name2.*</detail>.*<status>XML-INVALID</status>/s", "Returned OK status" );
}

# Check username/password mismatch
{
    my $req = HTTP::Request->new(
        "PUT" => "http://$hostname/api",
        [qw(Content-Type text/xml)],
        qq{<opsview><authentication>
			<username>nonadmin</username>
			<password>wrong</password>
			</authentication>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			</host>
			</opsview>
			},
    );
    ok( my $res = $ua->request($req), "Request" );
    ok( $res->is_error, "Error" );
    is( $res->code, 401, "Status 401" );
    like( $res->content, "/Username or password mismatch.*NO-ACCESS/s", "Expected error text" );
}

# Check can use credentials in XML data, with non admin user
{
    my $req = HTTP::Request->new(
        "PUT" => "http://$hostname/api",
        [qw(Content-Type text/xml)],
        qq{<opsview><authentication>
			<username>nonadmin</username>
			<password>nonadmin</password>
			</authentication>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			</host>
			</opsview>
			},
    );
    ok( my $res = $ua->request($req), "Request" );
    ok( $res->is_error, "Error" );
    is( $res->code, 401, "Status 401" );
    like( $res->content, "/Insufficient permissions.*NO-ACCESS/s", "text error message" );
}

# Check same error with login info via query string
{
    my $req = HTTP::Request->new(
        "PUT" => "http://$hostname/api?login_username=nonadmin&login_password=nonadmin",
        [qw(Content-Type text/xml)],
        qq{<opsview>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			</host>
			</opsview>
			},
    );
    ok( my $res = $ua->request($req), "Request" );
    ok( $res->is_error, "Error" );
    is( $res->code, 401, "Status 401" );
    like( $res->content, "/Insufficient permissions.*NO-ACCESS/s", "text error message" );
}

# Delete failure
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="delete" by_id="1">
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Cannot delete or update a parent row: a foreign key constraint fails.*FAILED/s", "Expected XML" );
}

# Delete for no such host by id
{
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="delete" by_id="1234567">
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, '/Cannot find host \(in Opsview::CRUD::Host\) with id 1234567.*OPTIONS-ERROR/s', "Expected XML" );
}

# Delete for no such host by name
{
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="delete" by_name="nosuchhost">
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Cannot find host with name nosuchhost.*OPTIONS-ERROR/s", "Expected XML" );
}

# Good delete by id
{
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="delete" by_id="2" />
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    is( $res->content, qq{<opsview>
  <result>
    <detail></detail>
    <status>OK</status>
  </result>
</opsview>
}, "Expected XML"
    );
}

# Missing fields
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Field ip is mandatory.*FAILED/s", "Expected XML" );
}

# Good create
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<name>testviaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>10.11.12.13</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			<icon><name>LOGO - Opsview</name></icon>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

# Good create via clone
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<clone><name>testviaxml</name></clone>
			<name>cloned</name>
			<snmp_version>4</snmp_version>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

# Good create via clone using id
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<clone><id>3</id></clone>
			<name>cloned2</name>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

# Good create with IPv6 address
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<name>testipv6viaxml</name>
			<check_command><name>ping</name></check_command>
			<ip>::1</ip> <hostgroup><name>Monitoring Servers</name></hostgroup>
			<icon><name>LOGO - Opsview</name></icon>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/OK/s", "Expected XML" );
}

# Failed create via clone with duplicate name
{
    my $res = This->test(
        HTTP::Request->new(
            "PUT" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="create">
			<clone><name>testviaxml</name></clone>
			<name>opsview</name>
			</host>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Duplicate entry 'opsview' for key .*FAILED/s", "Expected XML" );
}

# Good delete by name
{
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<host action="delete" by_name="testviaxml" />
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    is( $res->content, qq{<opsview>
  <result>
    <detail></detail>
    <status>OK</status>
  </result>
</opsview>
}, "Expected XML"
    );
}

# Disable notifications
{
    diag("Disable hostgroup notifications") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <notifications>disable</notifications>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    is( $res->content, qq{<opsview>
  <result>
    <detail></detail>
    <status>OK</status>
  </result>
</opsview>
}, "Expected XML"
    );
}

# Enable notifications
{
    diag("Enable hostgroup notifications") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <notifications>enable</notifications>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    is( $res->content, qq{<opsview>
  <result>
    <detail></detail>
    <status>OK</status>
  </result>
</opsview>
}, "Expected XML"
    );
}

# Rubbish notifications
{
    diag("Rubbish hostgroup notifications") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <notifications>rubbish</notifications>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Element notifications failed to validate content.*XML-INVALID/s", "XML right" );
}

# Enable hostgroup downtime - natural language times
{
    diag("Enable hostgroup downtime (natural language)") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
       <downtime
        start="now"
        end="in 2 hours"
        comment="Testing scheduled downtime"
       >enable</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
}

# Disable hostgroup downtime
{
    diag("Disable hostgroup downtime") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <downtime>disable</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
}

# Enable hostgroup downtime - in the past with natural language times
{
    diag("Enable hostgroup downtime in the past (natural language)") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
       <downtime
        start="2 hours ago"
        end="now "
        comment="Testing scheduled downtime"
       >enable</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/<detail>Start time is too far behind in time</detail>.*<status>FAILED</status>/s", "Expected XML" );
}

# Disable hostgroup downtime
{
    diag("Disable hostgroup downtime") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <downtime>disable</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
}

# Enable hostgroup downtime - standard dates & times
{
    my $start_epoc = time;
    my $end_epoc   = $start_epoc + 300;
    my $start      = strftime "%F %H:%M:%S", localtime($start_epoc);
    my $end        = strftime "%F %H:%M:%S", localtime($end_epoc);
    {
        diag("Enable hostgroup downtime (standard spec)") if ( $ENV{TEST_VERBOSE} );
        my $res = This->test(
            HTTP::Request->new(
                "POST" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<hostgroup action="change" by_name="UK">
				 <downtime
					start="$start"
					end="$end"
					comment="Testing scheduled downtime"
				 >enable</downtime>
				</hostgroup>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Success" );
        is( $res->code, 200, "Status 200" );
        like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
    }

    # Disable hostgroup downtime
SKIP: {
        skip "functionality not fully implemented", 3;

        diag("Disable specific hostgroup downtime") if ( $ENV{TEST_VERBOSE} );
        my $res = This->test(
            HTTP::Request->new(
                "POST" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<hostgroup action="change" by_name="UK">
				 <downtime
				 	start="$start"
					end="$end"
					comment="Testing scheduled downtime"
				 >disable</downtime>
				</hostgroup>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Success" );
        is( $res->code, 200, "Status 200" );
        like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
    }

    {
        diag("Disable hostgroup downtime") if ( $ENV{TEST_VERBOSE} );
        my $res = This->test(
            HTTP::Request->new(
                "POST" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<hostgroup action="change" by_name="UK">
				 <downtime>disable</downtime>
				</hostgroup>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Success" );
        is( $res->code, 200, "Status 200" );
        like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Expected XML" );
    }
}

# Enable hostgroup downtime - standard dates & times but in the past
{
    diag("Enable hostgroup downtime in the past (standard spec)") if ( $ENV{TEST_VERBOSE} );

    my $start_epoc = time - 4200;
    my $end_epoc   = time;
    my $start      = strftime "%F %H:%M:%S", localtime($start_epoc);
    my $end        = strftime "%F %H:%M:%S", localtime($end_epoc);
    my $comment    = "Testing scheduled downtime";

    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <downtime
				start="$start"
				end="$end"
				comment="$comment"
			 >enable</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Response error" );
    is( $res->code, 400, "status 400" );
    like( $res->content, "/<detail>Start time is too far behind in time</detail>.*<status>FAILED</status>/s", "Returned expected XML" );
}

{
    my $start_epoc = time;
    my $end_epoc   = time + 4200;
    my $start      = strftime "%F %H:%M:%S", localtime($start_epoc);
    my $end        = strftime "%F %H:%M:%S", localtime($end_epoc);
    my $comment    = "Testing scheduled downtime";
    {
        diag("Enable hostgroup downtime correctly (standard spec)") if ( $ENV{TEST_VERBOSE} );

        my $res = This->test(
            HTTP::Request->new(
                "POST" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<hostgroup action="change" by_name="UK">
				 <downtime
					start="$start"
					end="$end"
					comment="$comment"
				 >enable</downtime>
				</hostgroup>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Response success" );
        is( $res->code, 200, "status 200" );
        like( $res->content, "/<detail></detail>.*<status>OK</status>/s", "Returned expected XML" );
    }

    # junk downtime disable (no comment)
SKIP: {
        skip "functionality not fully implemented", 6;

        {
            diag("Disable hostgroup downtime (no comment)") if ( $ENV{TEST_VERBOSE} );
            my $res = This->test(
                HTTP::Request->new(
                    "POST" => "http://$hostname/api",
                    [qw(Content-Type text/xml)],
                    qq{<opsview>
					<authentication><username>admin</username><password>initial</password></authentication>
					<hostgroup action="change" by_name="UK">
					 <downtime
						 start="$start"
						 end="$end"
					 >rubbish</downtime>
					</hostgroup>
					</opsview>
					},
                )
            );
            ok( $res->is_error, "Error" );
            is( $res->code, 400, "Status 400" );
            like( $res->content, "/Element downtime failed to validate content.*XML-INVALID/s", "XML right" );
        }

        {
            diag("Disable hostgroup downtime (no end time)") if ( $ENV{TEST_VERBOSE} );
            my $res = This->test(
                HTTP::Request->new(
                    "POST" => "http://$hostname/api",
                    [qw(Content-Type text/xml)],
                    qq{<opsview>
					<authentication><username>admin</username><password>initial</password></authentication>
					<hostgroup action="change" by_name="UK">
					 <downtime
						 start="$start"
						 comment="$comment"
					 >rubbish</downtime>
					</hostgroup>
					</opsview>
					},
                )
            );
            ok( $res->is_error, "Error" );
            is( $res->code, 400, "Status 400" );
            like( $res->content, "/Element downtime failed to validate content.*XML-INVALID/s", "XML right" );
        }
    }

    {
        diag("Disable hostgroup downtime") if ( $ENV{TEST_VERBOSE} );
        my $res = This->test(
            HTTP::Request->new(
                "POST" => "http://$hostname/api",
                [qw(Content-Type text/xml)],
                qq{<opsview>
				<authentication><username>admin</username><password>initial</password></authentication>
				<hostgroup action="change" by_name="UK">
				 <downtime>disable</downtime>
				</hostgroup>
				</opsview>
				},
            )
        );
        ok( $res->is_success, "Success" );
        is( $res->code, 200, "Status 200" );
        like( $res->content, '/<detail></detail>.*<status>OK</status>/s', "Returned expected XML" );
    }
}

# Junk hostgroup downtime
{
    diag("Enable hostgroup downtime (junk)") if ( $ENV{TEST_VERBOSE} );
    my $res = This->test(
        HTTP::Request->new(
            "POST" => "http://$hostname/api",
            [qw(Content-Type text/xml)],
            qq{<opsview>
			<authentication><username>admin</username><password>initial</password></authentication>
			<hostgroup action="change" by_name="UK">
			 <downtime>rubbish</downtime>
			</hostgroup>
			</opsview>
			},
        )
    );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    like( $res->content, "/Element downtime failed to validate content.*XML-INVALID/s", "XML right" );
}

# Can't really test a failed reload here because at api level, it shouldn't occur
