#!perl

# TODO: Should create an expected array as a hash. Then get XML and JSON versions of the data
# and convert back again into perl structures and see whether it matches the hash. This would be
# a better, and more robust way, than trying to parse the output

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";

use Opsview::Web::Test;

use Test::More qw(no_plan);
use Test::LongString;

use HTTP::Request::Common;
use HTTP::Request;
use POSIX qw( strftime );

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc";

use XML::LibXML;
use Opsview::Test::FileContents;
use Test::JSON::File;

use JSON qw(decode_json encode_json);

my $parser = XML::LibXML->new;
my $rngschema = XML::LibXML::RelaxNG->new( location => "/usr/local/nagios/share/xml/opsview.rng" );

use Sys::Hostname;

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

use LWP::UserAgent;
my $ua = LWP::UserAgent->new;

# helper functions
# NOTE: some regexp constructs may need to be escaped in expected XML,
# i.e. ()'s
sub compare_lines {
    warn( '=============================================', $/ );
    my @res = split( /^/, shift );
    my @exp = split( /^/, shift );

    for ( my $i = 0; $i < $#res; $i++ ) {
        $res[$i] =~ m/$exp[$i]/ || warn( "LINE $i DIFFERS:\n", $res[$i], $exp[$i] );

        #like_string( $res[$i], qr/$exp[$i]/ ) || warn($res[$i],$exp[$i]);
    }
}

sub strip_xml {
    $_ = shift;
    s/state_duration="[0-9]+"//g;
    $_;
}

# No username/password
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request without username/password" );
    ok( $res->is_error, "Failed" );
    is( $res->code, 403, "Status 403" );
    like( $res->content, "/You are not authorised to use the status API/s", "Expected error message" );
}

# Get status information for service
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request for service totals" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );

    my $xml = strip_xml($res->content);
    #open F, "> $Bin/var/xml/service_full";
    #print F $xml;
    #close F;
    file_contents_compare( $xml, "$Bin/var/xml/service_full", "Expected XML" );

    # Check that asuser returns different data
    $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?asuser=somehosts" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( $res = $ua->request($r), "Request for service totals" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    $xml = strip_xml($res->content);
    file_contents_differ( $xml, "$Bin/var/xml/service_full", "Different XML for somehosts user" );

    # Check bad asuser
    $r = HTTP::Request->new( GET => "http://$hostname/api/status/service?asuser=somehosts999" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( $res = $ua->request($r), "Request for service totals" );
    ok( $res->is_error, "Error" );
    is( $res->code, 400, "Status 400" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    like_string( $res->content, qr/Contact somehosts999 not found/, "Expected XML" )
}

my $expected = qq{<opsview>
  <data name="hostgroup">
    <list name="UK" computed_state="critical" downtime="2" hostgroup_id="3">
      <hosts handled="9" total="10" unhandled="1">
        <down handled="1" unhandled="1" />
        <up handled="8" />
      </hosts>
      <services computed_state="critical" handled="6" total="23" unhandled="17">
        <critical unhandled="2" />
        <ok handled="3" />
        <unknown handled="3" unhandled="15" />
      </services>
    </list>
    <list name="UK2" computed_state="critical" downtime="1" hostgroup_id="6">
      <hosts handled="1" total="1" unhandled="0">
        <up handled="1" />
      </hosts>
      <services computed_state="critical" handled="0" total="3" unhandled="3">
        <critical unhandled="1" />
        <unknown unhandled="2" />
      </services>
    </list>
    <summary handled="16" total="37" unhandled="21">
      <host down="2" handled="10" total="11" unhandled="1" up="9" />
      <service critical="3" handled="6" ok="3" total="26" unhandled="20" unknown="20" />
    </summary>
  </data>
</opsview>
};

# Get status information for hostgroups
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/hostgroup" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request hostgroup hierarchy" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    like_string( $res->content, qr/$expected/s, "Expected XML" )
        || compare_lines( $res->content, $expected );
}

# Should be same as above
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/hostgroup/1" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request hostgroup hierarchy for 1" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    like_string( $res->content, qr/$expected/s, "Expected XML" )
        || compare_lines( $res->content, $expected );

    $r = HTTP::Request->new( GET => "http://$hostname/api/status/hostgroup/1?asuser=somehosts" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( $res = $ua->request($r), "Request hostgroup hierarchy for 1" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    unlike_string( $res->content, qr/$expected/s, "Expected XML" )
        || compare_lines( $res->content, $expected );
}

# Invalid hostgroup
{
    my $expected = qq{<opsview>
  <data error="Host group id 1234567890 not found" />
</opsview>
};
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/hostgroup/1234567890" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request invalid hostgroup hierarchy" );
    ok( $res->is_error, "Error from request" );
    is( $res->code, 404, "Status 404" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    is( $res->content,      $expected,  "Expected error XML" );
}

# Return data in json
{
    my $expected = '{"hostgroup":{"summary":{"handled":16,"unhandled":21,"total":37,"service":{"critical":3,"ok":3,"handled":6,"unknown":20,"unhandled":20,"total":26},"host":{"handled":10,"unhandled":1,"down":2,"up":9,"total":11}},"list":[{"computed_state":"critical","hostgroup_id":"3","hosts":{"handled":9,"unhandled":1,"down":{"handled":1,"unhandled":1},"up":{"handled":8},"total":10},"services":{"ok":{"handled":3},"critical":{"unhandled":2},"handled":6,"computed_state":"critical","unknown":{"handled":3,"unhandled":15},"unhandled":17,"total":23},"name":"UK","downtime":2},{"computed_state":"critical","hostgroup_id":"6","hosts":{"handled":1,"unhandled":0,"up":{"handled":1},"total":1},"services":{"critical":{"unhandled":1},"handled":0,"computed_state":"critical","unknown":{"unhandled":2},"unhandled":3,"total":3},"name":"UK2","downtime":1}]}}';
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/hostgroup" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "application/json" );
    ok( my $res = $ua->request($r), "Request for json" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );

    # Take this test out as the text may not be exactly the same, but below will confirm
    #is( $res->content, $expected, "Expected JSON" );

    my $expected_perl = decode_json($expected);
    my $got_perl      = decode_json( $res->content );
    is_deeply( $got_perl, $expected_perl, "JSON translates into perl structures correctly" );
}

# Return data in json for service page
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/service" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "application/json" );
    ok( my $res = $ua->request($r), "Request for service info via json" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );

    my $got_perl      = decode_json( $res->content );
    null_state_duration( $got_perl->{service} );
    my $got_json = encode_json( $got_perl );
    is_json_file( $got_json, "$Bin/var/json/service_full" );
}

# Return data in json for viewports page
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/viewport/cisco" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "application/json" );
    ok( my $res = $ua->request($r), "Request for viewport info via json" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );

    my $got_perl      = decode_json( $res->content );
    null_state_duration( $got_perl->{service} );
    my $got_json = encode_json( $got_perl );
    is_json_file( $got_json, "$Bin/var/json/viewport_cisco" );
}

# Get status information for viewport
{
    my $r = HTTP::Request->new( GET => "http://$hostname/api/status/viewport/cisco" );
    $r->header( "X-Username"   => "admin" );
    $r->header( "X-Password"   => "initial" );
    $r->header( "Content-Type" => "text/xml" );
    ok( my $res = $ua->request($r), "Request viewport data" );
    ok( $res->is_success, "Success" );
    is( $res->code, 200, "Status 200" );
    eval { $_ = $rngschema->validate( $parser->parse_string( $res->content ) ) };
    is( $_,                 0,          "Result XML valid" );
    is( $@,                 "",         "No eval error" );
    is( $res->content_type, "text/xml", "XML returned" );
    my $xml = strip_xml($res->content);
    file_contents_compare( $xml, "$Bin/var/xml/viewport_cisco", "Expected XML" );
}

sub null_state_duration {
    my $hash = shift;
    foreach my $item ( @{ $hash->{list} } ) {
        $item->{state_duration} = "" if exists $item->{state_duration};
        my @subitems;
        push @subitems, @{ $item->{services} } if exists $item->{services};
        push @subitems, @{ $item->{hosts} }    if exists $item->{hosts};
        foreach my $subitem (@subitems) {
            $subitem->{state_duration} = "" if exists $subitem->{state_duration};
        }
    }
}
