#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test qw(opsview);
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

use JSON;

my $content;
my $expected;
my $ua   = Test::WWW::Mechanize->new( stack_depth => 0 );
$ua->add_header( 'content-type' => "text/x-data-dumper" );
my $json = Test::WWW::Mechanize->new( stack_depth => 0 );
$json->add_header( 'content-type' => "application/json" );
my $url_prefix="http://localhost:3000";

$ua->get_ok("$url_prefix/rest");
is( $ua->status, 200 );
# Don't do a content test - this is done in t/650-api2-version.t

$ua->get("$url_prefix/rest/bobby");
is( $ua->status, 404, "Got 404 not found for invalid request" );
is( $ua->content, "{'message' => 'Request not found for /rest/bobby'}" );

$ua->get("$url_prefix/rest/info");
is( $ua->status, 401, "Unauthorised read of /rest/info" );
is( $ua->content, "{'message' => 'Token invalid'}" );

$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/login") );
is( $ua->status, 401, "Cannot DELETE session token as not authenticated");
is( $ua->content, "{'message' => 'Token invalid'}" );

$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "fake", password => "stuff" }%, "Failed login to API2" );
$ua->content_contains( "Invalid login", "Invalid login" );
is( $ua->status, 401, "Unauthorized" );
$content = $ua->content;
is( $content, qq%{'message' => 'Invalid login'}%, 'Login error message' );

$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "admin", password => "initial" }%, "Login to API2" );
is( $ua->status, 200 );
$ua->content_contains( "token", "Should return some ticket information" );
$content = $ua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got expected content" );
my $data = eval $content;
like( $data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

# Have to use post. Cannot use post_ok as this converts Content into a request parameter
$json->post( "$url_prefix/rest/login", Content => qq%{"username":"admin","password":"initial"}% );
is( $json->status, 200, "Can login correctly" );
$content = $json->content;
like( $content, qr%^\{"token":"[0-9a-f]{40}"\}$%, "Got JSON content" );

$json->post( "$url_prefix/rest/login?include_one_time_token=1", Content => qq%{"username":"admin","password":"initial"}% );
is( $json->status, 200, "Can login correctly" ) || die("Content: ".$json->content);
$content = decode_json($json->content);
#like( $content, qr%^\{"token":"[0-9a-f]{40}"\}$%, "Got JSON content" ) || diag("Content: ".$json->content);
like( $content->{token}, qr/[0-9a-f]{40}/, "Got 40 char token" );
like( $content->{one_time_token}, qr/[0-9a-f]{40}/, "Got 40 char one time token" );

$json->add_header( 'X-Opsview-Username' => "admin" );
$json->add_header( 'X-Opsview-Token' => $content->{token} );

my $testapi = Test::WWW::Mechanize->new( stack_depth => 0 );
$testapi->add_header( 'content-type' => "text/x-data-dumper" );
$testapi->add_header( "X-Opsview-Username" => "admin" );
$testapi->add_header( "X-Opsview-Token"    => $content->{one_time_token} );
$testapi->get( "$url_prefix/rest/info" );
is( $testapi->status, 401, "Cannot use token" );
my $resdata = eval $testapi->content;
is( $resdata->{message}, "One time token cannot be used for REST API", "Correct error message" );


my $browser = Test::WWW::Mechanize->new( stack_depth => 0 );
$browser->get_ok("$url_prefix/login?login_username=badusername&one_time_token=$content->{one_time_token}", "Should fail due to wrong username" );
$browser->content_contains("Username or password mismatch", "Token mismatch with username" );

$browser->get_ok("$url_prefix/login?login_username=admin&one_time_token=$content->{one_time_token}", "Can login okay with one time token" );
$browser->content_contains("Host Group Summary", "Logged in with one time token" ) || diag("Content: ". $browser->content);
$browser->get_ok("$url_prefix/logout", "Logout" );
$browser->content_contains("Please log in", "Logged out" );

$browser->get_ok("$url_prefix/login?login_username=admin&one_time_token=$content->{one_time_token}", "Should fail with this token now" );
$browser->content_contains("Username or password mismatch", "Token failed" );


$ua->add_header( "X-Opsview-Username" => "fakeuser" );
$ua->add_header( "X-Opsview-Token"    => $data->{token} );
$ua->get( "$url_prefix/rest/config/contact" );
is( $ua->status, 401, "Username doesn't match" );
$ua->content_contains( 'Token does not match username', "Correct error message" );

$ua->add_header( "X-Opsview-Username" => "admin" );
$ua->add_header( "X-Opsview-Token"    => "faketoken" );
$ua->get( "$url_prefix/rest/config/contact" );
is( $ua->status, 401, "Fake token" );
$ua->content_contains( 'Token invalid', "Correct error message" );

# Test capitalisation checks
$ua->add_header( "X-Opsview-Username" => "AdMin" );
$ua->add_header( "X-Opsview-Token"    => $data->{token} );

$ua->get_ok( "$url_prefix/rest/config/contact?cols=description,id,name,realm,role,fullname", "Get list of contacts" );
my $res_data = eval $ua->content;
$expected = {
    summary => {
        page => 1,
        rows => 14,
        totalpages => 1,
        totalrows => 14,
        allrows => 14,
    },
    'list' => [
        {   'description'              => 'Initial admin user',
            'id'                   => 1,
            'name'                 => 'admin',
            'realm'                => 'local',
            'role'                 => {
                'name' => 'Admin',
                'ref'  => '/rest/config/role/10',
            },
            'fullname' => 'admin',
            'ref'      => '/rest/config/contact/1',
        },
        {   'description'              => '',
            'id'                   => 12,
            'fullname'                 => 'Adminnoc with , comma',
            'realm'                => 'local',
            'role'                 => {
                'name' => 'Admin no configurehosts',
                'ref'  => '/rest/config/role/16',
            },
            'name' => 'adminnoc',
            'ref'      => '/rest/config/contact/12',
        },
        {   'description'              => '',
            'id'                   => 13,
            'name'                 => 'demo',
            'realm'                => 'local',
            'role'                 => {
                'name' => 'demorole',
                'ref'  => '/rest/config/role/17',
            },
            'fullname' => 'demo',
            'ref'      => '/rest/config/contact/13',
        },
        {   'description'              => '',
            'id'                   => 14,
            'name'                 => 'keywordonly',
            'realm'                => 'local',
            'role'                 => {
                'name' => 'Keywordonly',
                'ref'  => '/rest/config/role/21',
            },
            'fullname' => 'keywordonly',
            'ref'      => '/rest/config/contact/14',
        },
        {   'description'              => '',
            'id'                   => 2,
            'name'                 => 'nonadmin',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change some',
                'ref'  => '/rest/config/role/12',
            },
            'fullname' => 'nonadmin',
            'ref'      => '/rest/config/contact/2',
        },
        {   'description'              => 'Bug with unknowns',
            'id'                   => 5,
            'fullname'                 => 'only unknowns',
            'realm'                => 'local',
            'role'                 => {
                'name' => 'View some, change some',
                'ref'  => '/rest/config/role/12',
            },
            'name' => 'onlyunknowns',
            'ref'      => '/rest/config/contact/5',
        },
        {   'description'              => '',
            'id'                   => 3,
            'name'                 => 'readonly',
            'realm' => 'local',
            'role'  => {
                'name' => 'View all, change none',
                'ref'  => '/rest/config/role/13',
            },
            'fullname' => 'readonly',
            'ref'      => '/rest/config/contact/3',
        },
        {   'description'              => '',
            'id'                   => 4,
            'name'                 => 'somehosts',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change some - somehosts',
                'ref'  => '/rest/config/role/18',
            },
            'fullname' => 'somehosts',
            'ref'      => '/rest/config/contact/4',
        },
        {   'description'              => '',
            'id'                   => 7,
            'name'                 => 'testviewallchangenone',
            'realm' => 'local',
            'role'  => {
                'name' => 'View all, change none',
                'ref'  => '/rest/config/role/13',
            },
            'fullname' => 'testviewallchangenone',
            'ref'      => '/rest/config/contact/7',
        },
        {   'description'              => '',
            'id'                   => 6,
            'name'                 => 'testviewallchangesome',
            'realm' => 'local',
            'role'  => {
                'name' => 'View all, change some',
                'ref'  => '/rest/config/role/11',
            },
            'fullname' => 'testviewallchangesome',
            'ref'      => '/rest/config/contact/6',
        },
        {   'description'              => 'Only some keywords are visible',
            'id'                   => 10,
            'name'                 => 'viewkeywords',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change none',
                'ref'  => '/rest/config/role/14',
            },
            'fullname' => 'viewkeywords',
            'ref'      => '/rest/config/contact/10',
        },
        {   'description'              => '',
            'id'                   => 11,
            'name'                 => 'viewkeywords2',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change none, no notify',
                'ref'  => '/rest/config/role/15',
            },
            'fullname' => 'viewkeywords2',
            'ref'      => '/rest/config/contact/11',
        },
        {   'description'              => 'View some, change none',
            'id'                   => 8,
            'name'                 => 'viewsomechangenone',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change none - viewsomechangenone',
                'ref'  => '/rest/config/role/19',
            },
            'fullname' => 'viewsomechangenone',
            'ref'      => '/rest/config/contact/8',
        },
        {   'description'              => 'View some, change none, with no notifications',
            'id'                   => 9,
            'name'                 => 'viewsomechangenonewonotify',
            'realm' => 'local',
            'role'  => {
                'name' => 'View some, change none, no notify - viewsomechangenonewonotify',
                'ref'  => '/rest/config/role/20',
            },
            'fullname' => 'viewsomechangenonewonotify',
            'ref'      => '/rest/config/contact/9',
        }
    ]
};
is( $ua->status, 200, "Got list of contacts" );
is_deeply( $res_data, $expected, "Got expected data for contact list" ) || diag("Full dump: ".Data::Dump::dump($res_data));

$ua->get( "$url_prefix/rest/config/contact/100000" );
is( $ua->status, 404, "No object" );
$ua->content_contains( "No object found with id=100000", "Got error message" );

$ua->get_ok( "$url_prefix/rest/config/contact/1", "Get admin contact info" );
$expected = {
    object => {
        description           => "Initial admin user",
        encrypted_password => '$apr1$dEX5UaQz$D6fCyI197d33YcQQqe1Yj.',
        id                => 1,
        language             => "",
        fullname              => "admin",
        notificationprofiles => [],
        realm                => "local",
        role                 => {
            name => "Admin",
            ref  => "/rest/config/role/10",
        },
        uncommitted => 0,
        name  => "admin",
        variables => [ { name => "RSS_COLLAPSED", value => 1 }, { name => "RSS_MAXIMUM_AGE", value => 1440 }, { name => "RSS_MAXIMUM_ITEMS", value => 30 }, ],
    }
};
$res_data = eval $ua->content;
is_deeply( $res_data, $expected, "Got hash data for single contact as expected" );

$json->put( "$url_prefix/rest/config/contact/1", Content => qq%{"bad":"content","missing":"bracket"% );
is( $json->status, 400, "PUT failed due to bad json content" );
$json->content_contains( "Content-Type application/json had a problem with your request", "Error string found" );
$json->content_contains( "} expected while parsing object/hash" );

$json->get_ok("$url_prefix/rest/config/host?s.name=opsview&cols=id&rows=1");
is( $json->content, qq%{"summary":{"page":"1","totalrows":"1","allrows":"24","totalpages":"1","rows":"1"},"list":[{"ref":"/rest/config/host/1","id":"1"}]}%, "Got expected json with strings" );

$ua->put( "$url_prefix/rest/config/contact/1", content => '' );
is( $ua->status, 400, "PUT failed due to bad content" );
$ua->content_contains( "No update data found", "Error string found" ) || diag("Content: ".$ua->content);

$ua->put( "$url_prefix/rest/config/contact/1", content => '{ "broken" => "syntax"' );
is( $ua->status, 400, "PUT failed due to bad content" );
$ua->content_contains( "Content-Type text/x-data-dumper had a problem with your request", "Error string found" ) || diag($ua->content);
$ua->content_contains( "Missing right curly or square bracket", "More detailed error found" ) || diag($ua->content);

$ua->put( "$url_prefix/rest/config/contact/1", content => '{ language => "en", description => "badrobot" }' );
is( $ua->status, 200, "PUT OK" );
$res_data                       = eval $ua->content;
$expected->{object}->{language} = "en";
$expected->{object}->{description}  = "badrobot";
$expected->{object}->{uncommitted} = 1; # This gets changed due to PUT
is_deeply( $res_data, $expected, "Got hash data for contact after a PUT to update" );

# Try and change the uncommitted flag
$expected->{object}->{uncommitted} = 0;
$expected->{object}->{language} = "fr";
$ua->put( "$url_prefix/rest/config/contact/1", content => Data::Dump::dump( $expected->{object} ) );
is( $ua->status, 200, "PUT with a lot of data OK" ) || diag("content: ".$ua->content);
$res_data = eval $ua->content;
$expected->{object}->{uncommitted} = 1;
is_deeply( $res_data, $expected, "Got hash data back with expected data, including uncommitted flag set" );

$ua->put( "$url_prefix/rest/config/contact/1", content => '{ name => "bad%char^s!" }' );
is( $ua->status, 400, "PUT failed" );
$res_data = eval $ua->content;
$expected = { message => "Error trying to synchronise object", };
is( $res_data->{message}, $expected->{message}, "Got right error message" );
like( $res_data->{detail}, qr/^DBIx::Class::Row::throw_exception.* Failed constraint on name for /, "Got same error (roughly)" );

$ua->put( "$url_prefix/rest/config/contact/1", content => '{ name => die("Some text") }' );
is( $ua->status, 400, "PUT failed" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error with input or output data", "Got right error message") || diag( $ua->content );
like( $res_data->{detail}, qr/'die' trapped by operation mask/, "Got right trapped die");

$ua->get_ok( "$url_prefix/rest/config/contact/11", "Try to get contact 11 before delete" );
is( $ua->status, 200, "Contact 11 found" );

$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/config/contact/11") );
is( $ua->status, 200, "Delete succeeded" );

note("Try to get contact 11 after delete");
$ua->get("$url_prefix/rest/config/contact/11");
is( $ua->status, 404, "No object - deleted!" );
$ua->content_contains( "No object found with id=11", "Got error message" );

$ua->post("$url_prefix/rest/config/contact");
is( $ua->status, 400, "Create failed due to no data" );
$ua->content_contains("No create data found");

# Need to set Content-Type otherwise WWW::Mechanize will assume post sends form data
$ua->add_header( 'Content-type' => "text/x-data-dumper" );
$ua->post( "$url_prefix/rest/config/contact", Content => '"a string value"' );
is( $ua->status, 400, "Data incorrect" );
$ua->content_contains("Data not in correct format");

my $creation = {
    fullname              => "New Body",
    name          => "newbody",
    role              => { name => "view some, change some" },
    language          => "it",
    all_hostgroups    => 0,
    all_servicegroups => 0,
    hostgroups        => [],
    servicegroups     => [],
    'password'        => "torso",
    variables         => [ { name => "FRED", value => "Flintstone" } ],
    notificationprofiles => [ { name => "profile1", notificationmethods => [ { name => "RSS" }, { name => "Email" } ] }, { name => "profile2" } ],
};
$expected = {
    'description'              => "",
    'language'             => 'it',
    'fullname'                 => 'New Body',
    'name'             => 'newbody',
    'notificationprofiles' => [
        { all_hostgroups =>1,
            all_servicegroups => 1,
            all_keywords => 1,
            host_notification_options => "n",
            hostgroups => [],   
            keywords => [],
            name => "profile1",
            notification_level => 1,
            notification_period => { name => "24x7", ref => "/rest/config/timeperiod/1" },
        notificationmethods => [
            { name => "Email", ref => "/rest/config/notificationmethod/3" },
            { name => "RSS", ref => "/rest/config/notificationmethod/4" },
        ],
        'ref' => '/rest/config/notificationprofile/14',
        service_notification_options => "n",
        servicegroups => [],
        uncommitted => 1,
        },{
            all_hostgroups => 1,
            all_servicegroups => 1,
            all_keywords => 1,
            host_notification_options => "n",
            hostgroups => [],
            keywords => [],
          name => "profile2",
            notification_level => 1,
            notification_period => { name => "24x7", ref => "/rest/config/timeperiod/1" },
        'ref' => '/rest/config/notificationprofile/15',
        service_notification_options =>"n",
        servicegroups => [],
        uncommitted => 1,
        notificationmethods => [],
        },
    ],
    'realm'                => 'local',
    'id'                   => '15',
    'role'                 => {
        'ref'  => '/rest/config/role/12',
        'name' => 'View some, change some'
    },
    variables => [ { name => "FRED", value => "Flintstone" } ],
    uncommitted => 1,
};
$ua->post( "$url_prefix/rest/config/contact", Content => Data::Dump::dump($creation) );
is( $ua->status, 200, "Create succeeded" );
$res_data = eval $ua->content;
# Can't test password because this changes
delete $res_data->{object}->{encrypted_password};
is_deeply( $res_data->{object}, $expected, "Expected return of data too" ) || diag("Full dump: ".Data::Dump::dump($res_data->{object}));

my $newbody = Test::WWW::Mechanize->new( stack_depth => 0 );
$newbody->add_header( 'content-type' => 'text/x-data-dumper' );
$newbody->post( "$url_prefix/rest/login", Content => qq%{ username => "newbody", password => "torso" }%, "Check can login with password" );
is( $newbody->status, 200, "Login with password specified good to go!" );

$creation = { fullname => "Missing name", };
$ua->post( "$url_prefix/rest/config/contact", Content => Data::Dump::dump($creation) );
is( $ua->status, 400, "Create failed" );
$res_data = eval $ua->content;
$expected = {
    message => "Error trying to create object",
    detail  => "name: Missing\n",
};
is_deeply( $res_data, $expected, "Expected error message" );

$ua->get_ok( "$url_prefix/rest/config/contact?rows=2&page=1&cols=fullname&cols=id&cols=name,role", "Get list of contacts" );
$res_data = eval $ua->content;
$expected = {
    'list' => [
        {   
            'id'                   => 1,
            'name'                 => 'admin',
            'role'                 => {
                'name' => 'Admin',
                'ref'  => '/rest/config/role/10',
            },
            'fullname' => 'admin',
            'ref'      => '/rest/config/contact/1',
        },
        {   
            'id'                   => 12,
            'fullname'                 => 'Adminnoc with , comma',
            'role'                 => {
                'name' => 'Admin no configurehosts',
                'ref'  => '/rest/config/role/16',
            },
            'name' => 'adminnoc',
            'ref'      => '/rest/config/contact/12',
        },
    ],
    summary => {
        page => 1,
        rows => 2,
        totalpages => 7,
        totalrows => 14,
        allrows => 14,
   },
};
is( $ua->status, 200, "Got list of contacts with selected columns" );
is_deeply( $res_data, $expected, "Got expected data for contact list" ) || diag("Full dump: ".Data::Dump::dump($res_data));

$ua->get_ok( "$url_prefix/rest/config/notificationmethod?rows=all&cols=name", "Get full list of notificationmethods" );
$res_data = eval $ua->content;
$expected = {
    'list' => [
        { name => "Email", ref => "/rest/config/notificationmethod/3" },
        { name => "unused", ref => "/rest/config/notificationmethod/5" },
        { name => "AQL", ref => "/rest/config/notificationmethod/1" },
        { name => "SMS Notification Module", ref => "/rest/config/notificationmethod/2" },
        { name => "RSS", ref => "/rest/config/notificationmethod/4" },
    ],
    summary => {
        rows => "5",
        allrows => 5,
   },
};
is( $ua->status, 200, "Got full list of notificationmethods" );
is_deeply( $res_data, $expected, "Got expected data for notificationmethods list" ) || diag("Full dump: ".Data::Dump::dump($res_data));

$ua->get_ok( "$url_prefix/rest/config/host?rows=0&cols=name", "Get summary of hosts" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        rows => 24,
        allrows => 24,
   },
};
is( $ua->status, 200, "Got full list of hosts" );
is_deeply( $res_data, $expected, "Got expected data for hosts list" ) || diag("Full dump: ".Data::Dump::dump($res_data));


$ua->post( "$url_prefix/rest/config/host", Content => '{list=>[{name=>"listhost1"},{name=>"listhost2"},{name=>"listhost3"}]}' );
is( $ua->status, 200, "Created extra hosts" );
$res_data = eval $ua->content;
is_deeply( $res_data, {objects_updated => 3}, "Got result" ) || diag(Data::Dump::dump($res_data));

$ua->get_ok( "$url_prefix/rest/config/host?rows=0&cols=name", "Check list of hosts incremented" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        rows => 27,
        allrows => 27,
   },
};
is( $ua->status, 200, "Got full list of hosts" );
is_deeply( $res_data, $expected, "Got expected data for hosts list" ) || diag("Full dump: ".Data::Dump::dump($res_data));


$ua->post( "$url_prefix/rest/config/host", Content => '{list=>[{name=>"listhost4"},{name=>"listhost2"},{name=>"listhost6"}]}' );
is( $ua->status, 400, "Failed at a host" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Rollback. Error trying to create object: 2", "Correct error message" );
like( $res_data->{detail}, qr/execute failed: Duplicate entry 'listhost2' for key/, "Correct detail" );
is( $res_data->{objects_updated}, 0, "No objects updated due to rollback transaction" );

$ua->get_ok( "$url_prefix/rest/config/host?rows=0&cols=name", "Check list of hosts incremented" );
$res_data = eval $ua->content;
$expected = {
    summary => {
        rows => 27,
        allrows => 27,
   },
};
is( $ua->status, 200, "Got full list of hosts" );
is_deeply( $res_data, $expected, "Got expected data for hosts list" ) || diag("Full dump: ".Data::Dump::dump($res_data));



$ua->post( "$url_prefix/rest/config/host/12", Content => '{name=>"clonedhost"}' );
$res_data = eval $ua->content;
is( $ua->status, 200, "Cloned host succeeded" );
is( $res_data->{object}->{alias}, "Host to clone, testing API", "Got some older data" ) || diag(Data::Dump::dump($res_data));
is( $res_data->{object}->{name}, "clonedhost", "Got newer information" );
isnt( $res_data->{object}->{id}, 12, "And not the same id as the host cloned from" );

$ua->post( "$url_prefix/rest/config/host/12", Content => '{name=>"clonedhost"}' );
$res_data = eval $ua->content;
is( $ua->status, 400, "Clone failed due to unique constraint on name" );
is( $res_data->{message}, "Error trying to create object", "Correct error message" );
like( $res_data->{detail}, qr/execute failed: Duplicate entry 'clonedhost' for key/, "Correct detail" );


$ua->post( "$url_prefix/rest/config/host", Content => '[]' );
is( $ua->status, 400, "Failed due to bad input data" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Data not in correct format", "Correct error message" );

$ua->post( "$url_prefix/rest/config/host", Content => '{object=>[]}' );
is( $ua->status, 400, "Failed due to bad input data" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Data not in correct format", "Correct error message" );

$ua->post( "$url_prefix/rest/config/host", Content => '{object=>{ name => "opsview", ip => "shouldnotcreate" }}' );
is( $ua->status, 400, "Failed due to foreign constraint - but did process 'object' key correctly" );
$res_data = eval $ua->content;
is( $res_data->{message}, "Error trying to create object", "Correct error message" );
like( $res_data->{detail}, qr/execute failed: Duplicate entry 'opsview' for key/, "Correct detail" );


$ua->put( "$url_prefix/rest/config/contact/2", Content => '{description => "fake", badattr => [ "inarraycausesfailure" ] }' );
$res_data = eval $ua->content;
is( $ua->status, 400, "Bad data as have an array of scalars" );
note(Data::Dump::dump($res_data));
is( $res_data->{message}, "Error when parsing data", "Correct error message" );
like( $res_data->{detail}, qr/Not a HASH: inarraycausesfailure/, "Correct detail" );



note("Test session removals");
$ua->request( HTTP::Request::Common::DELETE("$url_prefix/rest/login") );
is( $ua->status, 200, "Logout succeeded" );

$ua->get( "$url_prefix/rest/info" );
is( $ua->status, 401, "Authentication denied now due to session key deleted" );

1;
