#!/usr/bin/perl
# Test rest

use warnings;
use strict;
use FindBin qw($Bin);
use lib "$Bin/../lib";

use lib "/usr/local/nagios/perl/lib", "/usr/local/nagios/lib", "/usr/local/nagios/etc", "/usr/local/nagios/t/lib";

use Test::More qw(no_plan);
use Test::Deep;
use Test::LongString;

use Opsview::Test;
use Test::WWW::Mechanize;

use Data::Dump qw(dump);

my $res_data;
my $content;
my $expected;
# Use undef as poor ACLs combined with a saved cookie was allowing access
my $ua   = Test::WWW::Mechanize->new( stack_depth => 0, cookie_jar => undef );
$ua->add_header('content-type' => 'text/x-data-dumper' );
my $url_prefix="http://localhost:3000";

$ua->post( "$url_prefix/rest/login", Content => qq%{ username => "admin", password => "initial" }%, "Login to API2" );
is( $ua->status, 200 );
$ua->content_contains( "token", "Should return some ticket information" );
$content = $ua->content;
like( $content, qr%^\{'token' => '[0-9a-f]{40}'\}$%, "Got expected content" );
my $data = eval $content;
like( $data->{token}, qr/^[0-9a-f]{40}$/, "Got token value" );

$ua->add_header("X-Opsview-Username" => "admin");
$ua->add_header("X-Opsview-Token" => $data->{token});

$ua->get_ok( "$url_prefix/rest/reload" );
$res_data = eval $ua->content;
is( $res_data->{server_status}, 2, "Server not running" );
is( $res_data->{configuration_status}, "uptodate", "Changes up to date" );
like( $res_data->{lastupdated}, qr/^\d{10}$/ );
my $lastupdated = $res_data->{lastupdated};
is( $res_data->{average_duration}, 10);
is( $res_data->{auditlog_entries}, undef, "No backup, and so no auditlog entries yet" );
is( scalar @{$res_data->{messages}}, 0, "No messages because of fresh DB") || diag(Data::Dump::dump($res_data->{messages}));

my $old_reload = { %$res_data };



$ua->get_ok( "$url_prefix/rest/serverinfo", "Get nodestatus info along with reload info" );
$res_data = eval $ua->content;

# This is merged with the previous reload info
$expected = {
    reload => $old_reload,
    num_hosts => 24,
    num_monitoringservers => 4,
    num_nodes => 5,
    monitoringservers => [
    { 
      name => "Master Monitoring Server",
      nodes => [ {'status' => "2", name => "opsview", ip => "opsviewdev46", latency => undef } ],
      activated => 1,
    },
    { 
      name => "ClusterA",
      nodes => [ { 'status' => '0','name' => 'opslave', ip => "opslave", 'latency' => '0.158'} ],
      activated => 1,
    },
    { 
      name => "Cluster",
      nodes => [ 
        {'status' => '2', 'name' => 'opslaveclusterA', ip => "opslaveclusterAip", 'latency' => undef},
        {'status' => '3',  'name' => 'opslaveclusterB', ip => "opslaveclusterBip", 'latency' => undef},
        {'status' => '3',  'name' => 'opslaveclusterC', ip => "opslaveclusterCip", 'latency' => undef},
      ],
      activated => 1,
    },
    { 
      name => "PassiveSlave",
      nodes => [ { 'status' => '3','name' => 'PassiveSlave', ip => "192.168.105.15", 'latency' => undef} ],
      activated => 1,
    },
    ]
};
is_deeply( $res_data, $expected, "Node status as expected" ) || diag( dump($res_data) );

$ua->post("$url_prefix/rest/reload");
is( $ua->status, 400, "Reload error" ) || diag( "Full content: ".$ua->content);
$res_data = eval $ua->content;
is( $res_data->{server_status}, 2, "Server still down" );
is( $res_data->{configuration_status}, "uptodate" );
is( $res_data->{lastupdated}, $lastupdated );
is( $res_data->{average_duration}, 10);
is( $res_data->{auditlog_entries}, undef, "Still none here" );
isnt( scalar @{$res_data->{messages}}, 0, "Various errors in messages - check for non-zero") || diag(Data::Dump::dump($res_data->{messages}));

system("/usr/local/nagios/bin/rc.opsview start");

$ua->get_ok( "$url_prefix/rest/reload" );
$res_data = eval $ua->content;
is( $res_data->{server_status}, 3, "Critical errors" );
is( $res_data->{configuration_status}, "uptodate" );
is( $res_data->{lastupdated}, $lastupdated );
is( $res_data->{average_duration}, 10);
is( $res_data->{auditlog_entries}, undef );
isnt( scalar @{$res_data->{messages}}, 0, "Still have those errors") || diag(Data::Dump::dump($res_data->{messages}));

$ua->post("$url_prefix/rest/reload");
is( $ua->status, 400, "Reload stil fails" );
$res_data = eval $ua->content;
is( $res_data->{server_status}, 3, "Due to slaves that can't be connected" );
is( $res_data->{configuration_status}, "uptodate" );
is( $res_data->{lastupdated}, $lastupdated );
is( $res_data->{average_duration}, 10);
is( $res_data->{auditlog_entries}, undef );
isnt( scalar @{$res_data->{messages}}, 0, "Still have those errors") || diag(Data::Dump::dump($res_data->{messages}));

system("/usr/local/nagios/utils/test_db_post");

$ua->post("$url_prefix/rest/reload");
is( $ua->status, 200 );
$res_data = eval $ua->content;
is( $res_data->{server_status}, "4", "Some warnings for server status" );
is( $res_data->{configuration_status}, "uptodate", "Now configuration up to date" );
isnt( $res_data->{lastupdated}, $lastupdated, "Last update time should have changed" );
is( $res_data->{average_duration}, 10, "Duration is 10 seconds" );
# Do not check auditlog_entries, because this is likely to change while reading as backups are asynchronous
$expected = [
    {
        detail => "Missing required variable PAGER for contact somehosts for notification method AQL - ignoring this notification",
        monitoringserver => "Master Monitoring Server",
        severity => "warning",
    },
    {
        detail => "Unable to lookup IP address for 'cisco4' (not_a_real_host) - ignoring SNMP trap configuration for this host",
        monitoringserver => "Master Monitoring Server",
        severity => "warning",
    },
    {
        detail => "IP 192.168.10.22 has more than 1 host associated with it: cisco2, cisco3",
        monitoringserver => "Master Monitoring Server",
        severity => "warning",
    }
];
is_deeply( $res_data->{messages}, $expected, "Messages as expected" ) || diag explain $res_data->{messages};

note("Sleeping to allow backup to run");
sleep 45;

$ua->get_ok( "$url_prefix/rest/reload" );
$res_data = eval $ua->content;
is( $res_data->{auditlog_entries}, 0, "Number of auditlogs now reset" );
