#!perl

use warnings;
use strict;

use FindBin qw($Bin);
use lib "$Bin/lib";
use lib "/usr/local/nagios/lib", "/usr/local/nagios/t/lib";

use Test::WWW::Mechanize;
use HTTP::Cookies;
use Sys::Hostname;

use Opsview::Test qw(opsview);
use Opsview::Web::Test;

use Opsview;
use Opsview::Contact;

use Test::More qw(no_plan);
use Test::Deep;

my $hostname = Sys::Hostname::hostname;
$hostname = "localhost:3000";

# This holds their role id
# Roles for non CONFIGUREVIEW users do not show complete list of roles now
my $users = {
    nonadmin              => 12,
    testviewallchangesome => 11,
    testviewallchangenone => 13,
};

foreach my $username ( keys %$users ) {

    my $ua = Test::WWW::Mechanize->new();
    my $cj = HTTP::Cookies->new();
    $ua->cookie_jar($cj);

    # No cookies here - should get login page
    my $res;
    $res = $ua->get("http://$hostname/");
    ok( $res->is_success, "Got successful response" );
    is( $res->code, 200, "Status 200" );
    like( $res->content, "/login_username/s", "Returned login_username, so is a login page" );
    my $content = $res->content;
    $content =~ s%<noscript>.*?</noscript>%%s;    # Remove "JavaScript is required message"
    unlike( $content, "/<h3>/s", "Check that no login error is displayed" );

    cmp_deeply( $cj->{COOKIES}, {}, "Cookie jar is empty" );

    $ua->field( 'login_username', $username );
    $ua->field( 'login_password', $username );
    $ua->field( "noscript" => 0 );
    $ua->current_form->action;
    $ua->submit;

    ok( $ua->res->is_success, "Successful reply" );
    like( $ua->content, qr{<title>Opsview &gt; Host Group Summary &gt; Opsview</title>}, "Found heading for HH page" );

    # Check cookies created
    my $cookies_hash = $cj->{COOKIES}->{"localhost.local"}->{"/"};
    ok( defined $cookies_hash->{auth_tkt},            "auth_tkt cookie assigned" );
    ok( defined $cookies_hash->{opsview_web_session}, "opsview_web_session cookie assigned" );

    $ua->get("http://$hostname/user/preference");

    $ua->form_name("main_form");
    ok( $ua->res->is_success, "Got user preferences page" );
    unlike( $ua->content, '/myemail@dummy.com/', "Email not found" );
    is( $ua->value('fullname'), $username );
    is( $ua->value('name'),     $username );
    is( $ua->value('description'),  "" );
    is( $ua->value('variables[EMAIL]'),    "" );
    is( $ua->value('variables[PAGER]'),   "" );
    is( $ua->value('variables[RSS_MAXIMUM_ITEMS]'),      "30" );
    is( $ua->value('variables[RSS_MAXIMUM_AGE]'),        "1440" );
    is( $ua->value('variables[RSS_COLLAPSED]'),      "1" );
    is( $ua->value("password"),            "", "Blank password" );
    is( $ua->value("language"),            "", "Language as default" );

    $ua->field( "fullname",    "amended" );
    $ua->field( "variables[EMAIL]",   'myemail@dummy.com' );
    $ua->field( "variables[PAGER]",  '1234' );
    $ua->field( "description", "This is an allowed change too" );
    $ua->field( "variables[RSS_MAXIMUM_ITEMS]", 45 );
    $ua->field( "variables[RSS_MAXIMUM_AGE]",   2000 );
    $ua->field( "variables[RSS_COLLAPSED]", 0 );
    $ua->field( "password", "newpassword" );
    $ua->select( "language", "fr" );
    $ua->current_form->action;
    $ua->submit;

    ok( $ua->res->is_success, "Saved user preferences page" );
    like( $ua->content, "/Successfully saved/", "Saved message" );
    unlike( $ua->content, "/Nonadmin23/", "Name has not been changed" );

    $ua->form_name("main_form");
    is( $ua->value('name'), $username );
    is( $ua->value('fullname'),     "amended" );
    is( $ua->value('description'),  "This is an allowed change too" );
    is( $ua->value('variables[EMAIL]'),    'myemail@dummy.com' );
    is( $ua->value('variables[PAGER]'),   "1234" );
    is( $ua->value('variables[RSS_MAXIMUM_ITEMS]'),      "45" );
    is( $ua->value('variables[RSS_MAXIMUM_AGE]'),        "2000" );
    is( $ua->value('variables[RSS_COLLAPSED]'),      "1" );
    is( $ua->value("password"),            "", "Password not displayed" );
    is( $ua->value("language"),            "fr", "Language changed to French" );

    # There is currently no send_test_sms in trunk
    #$ua->get_ok("/user/preference/send_test_sms");
    #is( $ua->content, "No output. Exit code 127" );

    $ua->get_ok("/logout");

    $ua->get_ok("/");
    like( $ua->content, "/login_username/s", "Get login page here" );

    $ua->field( 'login_username', $username );
    $ua->field( 'login_password', "newpassword" );
    $ua->field( "noscript" => 0 );
    $ua->current_form->action;
    $ua->submit;

    unlike( $ua->content, "/login_username/s", "Should not get login page now" );

}

{

    # Test that a user cannot change the post form to change the admin user's password
    my $ua = Test::WWW::Mechanize->new( cookie_jar => {} );

    $ua->get_ok("http://$hostname/user/preference");
    $ua->field( 'login_username', "nonadmin" );
    $ua->field( 'login_password', "newpassword" );
    $ua->field( "noscript" => 0 );
    $ua->current_form->action;
    $ua->submit;

    $ua->content_unlike("/login_username/");

    $ua->post_ok( "http://$hostname/user/preference/save/1", { password => "hackable", description => "Hack attempt" } );

    my $crack = Test::WWW::Mechanize->new( cookie_jar => {} );
    $crack->get_ok("http://$hostname/");
    $crack->field( 'login_username', "admin" );
    $crack->field( 'login_password', "hackable" );
    $crack->field( "noscript" => 0 );
    $crack->current_form->action;
    $crack->submit;
    $crack->content_like("/login_username/");

}

my $ua = Test::WWW::Mechanize->new( cookie_jar => {} );

$ua->get_ok("http://$hostname/user/preference");
$ua->field( 'login_username', "viewkeywords2" );
$ua->field( 'login_password', "viewkeywords2" );
$ua->field( "noscript" => 0 );
$ua->current_form->action;
$ua->submit;

$ua->content_contains("Changes to notifications", "Proves that this user does not have CONFIGUREVIEW access") || diag( $ua->content );
$ua->content_lacks( "realm", "Should not have any realm information" );
$ua->content_lacks( "Admin no configurehosts", "Role information of others should not get listed" );

$ua->content_contains("No notifications are allowed due to your role", "Do not show any notification options because of no NOTIFYSOME");
$ua->content_lacks( "psearch_form", "Should not have a page search box" );
$ua->content_lacks("view_options_button", "Should not have a view_options_button either as notification profiles not allowed");
$ua->content_lacks("Submit and Edit Notification Profiles", "Do not have extra button correctly" );


my $somehosts = Test::WWW::Mechanize->new( cookie_jar => {} );

$somehosts->get_ok("http://$hostname/user/preference");
$somehosts->field( 'login_username', "somehosts" );
$somehosts->field( 'login_password', "somehosts" );
$somehosts->field( "noscript" => 0 );
$somehosts->current_form->action;
$somehosts->submit;

$somehosts->content_contains("Notification profiles: 2", "Got detail re: notification profiles" );
is( $somehosts->find_link( text => "Notification profiles: 2" )->url, "/user/notificationprofile", "Got link");
$somehosts->content_lacks("Create notification profiles", "Help text re: creating notification profiles missing correctly");
$somehosts->content_contains("Submit and Edit Notification Profiles", "Have extra button" );

my $onlyunknowns = Test::WWW::Mechanize->new( cookie_jar => {} );

$onlyunknowns->get_ok("http://$hostname/user/preference");
$onlyunknowns->field( 'login_username', "onlyunknowns" );
$onlyunknowns->field( 'login_password', "onlyunknowns" );
$onlyunknowns->field( "noscript" => 0 );
$onlyunknowns->current_form->action;
$onlyunknowns->submit;

$onlyunknowns->content_contains("Notification profiles: 0", "Got detail re: notification profiles" );
is( $onlyunknowns->find_link( text => "Notification profiles: 0" )->url, "/user/notificationprofile/new", "Got link");
$onlyunknowns->content_contains("Create notification profiles", "Help text re: creating notification profiles");




